#version 410
layout(location = 0) in vec3 vp;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 vn;
layout(location = 3) in vec3 tangent_ms;
layout(location = 4) in vec3 bitangent_ms;


uniform mat4 MVP;
//uniform mat4 V;
uniform mat4 MV;
uniform vec3 LightPosition_worldspace;
uniform vec3 eyePosition;

out vec2 UV;
out vec3 Position_worldspace;
//out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace;

out vec3 EyeDirection_tangentspace;
out vec3 LightDirection_tangentspace;

void main( void )
{
  
   
   	gl_Position =  MVP * vec4(vp,1);
	
	// Position of the vertex, in worldspace : M * position
	Position_worldspace = vp;//(M * vec4(vp,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexPosition_cameraspace = ( MV* vec4(vp,1)).xyz;
	EyeDirection_cameraspace =(vec3(0,0,0) - vertexPosition_cameraspace);

	vec3 tangentObject=normalize(tangent_ms);
	vec3 normal= normalize(vn);
	
	vec3 biNormal=cross(tangentObject, normal);
	vec3 viewDirection_ws=-eyePosition-Position_worldspace;
	vec3 lightDirection_ws=LightPosition_worldspace-Position_worldspace;
	
	EyeDirection_tangentspace.x=dot(tangentObject, viewDirection_ws);
	EyeDirection_tangentspace.y=dot(biNormal,viewDirection_ws);
	EyeDirection_tangentspace.z=dot(normal,viewDirection_ws);

	LightDirection_tangentspace.x=dot(tangentObject, lightDirection_ws);
	LightDirection_tangentspace.y=dot(biNormal,lightDirection_ws);
	LightDirection_tangentspace.z=dot(normal,lightDirection_ws);
	
	

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 LightPosition_cameraspace = ( MV * vec4(LightPosition_worldspace,1)).xyz;
	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;
	
	// UV of the vertex. No special space for this one.
	UV = vertexUV;
   
   
   
}